@@ -1,3 +1,6 @@
+//! decode module
+
+use core::convert::TryFrom;
 use crate::core::{ major, marker, types };
 pub use crate::error::DecodeError as Error;
 
@@ -5,31 +8,67 @@ pub use crate::error::DecodeError as Error;
 use alloc::{ vec::Vec, string::String };
 
 
-pub trait Read<'a> {
+/// Read trait
+///
+/// This is similar to `BufRead` of standard library,
+/// but can define its own error types, and can get a
+/// reference with a long enough lifetime to implement zero-copy decode.
+pub trait Read<'de> {
     #[cfg(feature = "use_std")]
     type Error: std::error::Error + 'static;
 
     #[cfg(not(feature = "use_std"))]
     type Error: core::fmt::Display + core::fmt::Debug;
 
-    fn fill<'b>(&'b mut self, want: usize) -> Result<Reference<'a, 'b>, Self::Error>;
+    /// Returns the available bytes.
+    ///
+    /// The want value is the expected value.
+    /// If the length of bytes returned is less than this value,
+    /// zero-copy decoding will not be possible.
+    ///
+    /// Returning empty bytes means EOF.
+    fn fill<'short>(&'short mut self, want: usize) -> Result<Reference<'de, 'short>, Self::Error>;
+
+    /// Advance reader
     fn advance(&mut self, n: usize);
 
+    /// Step count
+    ///
+    /// This method maybe called when the decode is started
+    /// to calculate the decode depth.
+    /// If it returns false, the decode will return a depth limit error.
+    #[inline]
     fn step_in(&mut self) -> bool {
         true
     }
 
+    /// Step count
+    ///
+    /// This method maybe called when the decode is completed
+    /// to calculate the decode depth.
+    #[inline]
     fn step_out(&mut self) {}
 }
 
-pub enum Reference<'a, 'b> {
-    Long(&'a [u8]),
-    Short(&'b [u8])
+/// Bytes reference
+pub enum Reference<'de, 'short> {
+    /// If the reader can return bytes as long as its lifetime,
+    /// then zero-copy decoding will be allowed.
+    Long(&'de [u8]),
+
+    /// Bytes returned normally
+    Short(&'short [u8])
 }
 
+/// Decode trait
 pub trait Decode<'a>: Sized {
+    /// Decode with first byte
+    ///
+    /// The first byte can be read in advance to determine the decode type.
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>>;
 
+    /// Decode to type
+    #[inline]
     fn decode<R: Read<'a>>(reader: &mut R) -> Result<Self, Error<R::Error>> {
         let byte = pull_one(reader)?;
         Self::decode_with(byte, reader)
@@ -47,7 +86,8 @@ impl Reference<'_, '_> {
 }
 
 #[inline]
-pub fn peek_one<'a, R: Read<'a>>(reader: &mut R) -> Result<u8, Error<R::Error>> {
+#[cfg_attr(not(feature = "serde1"), allow(dead_code))]
+pub(crate) fn peek_one<'a, R: Read<'a>>(reader: &mut R) -> Result<u8, Error<R::Error>> {
     let b = reader.fill(1)?
         .as_ref()
         .get(0)
@@ -57,7 +97,7 @@ pub fn peek_one<'a, R: Read<'a>>(reader: &mut R) -> Result<u8, Error<R::Error>>
 }
 
 #[inline]
-pub fn pull_one<'a, R: Read<'a>>(reader: &mut R) -> Result<u8, Error<R::Error>> {
+pub(crate) fn pull_one<'a, R: Read<'a>>(reader: &mut R) -> Result<u8, Error<R::Error>> {
     let b = reader.fill(1)?
         .as_ref()
         .get(0)
@@ -86,17 +126,19 @@ fn pull_exact<'a, R: Read<'a>>(reader: &mut R, mut buf: &mut [u8]) -> Result<(),
     Ok(())
 }
 
-struct TypeNum {
+pub(crate) struct TypeNum {
     major_limit: u8,
     byte: u8
 }
 
 impl TypeNum {
-    const fn new(major_limit: u8, byte: u8) -> TypeNum {
+    #[inline]
+    pub(crate) const fn new(major_limit: u8, byte: u8) -> TypeNum {
         TypeNum { major_limit, byte }
     }
 
-    fn decode_u8<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u8, Error<R::Error>> {
+    #[inline]
+    pub(crate) fn decode_u8<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u8, Error<R::Error>> {
         match self.byte & self.major_limit {
             x @ 0 ..= 0x17 => Ok(x),
             0x18 => pull_one(reader),
@@ -105,6 +147,7 @@ impl TypeNum {
     }
 
 
+    #[inline]
     fn decode_u16<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u16, Error<R::Error>> {
         match self.byte & self.major_limit {
             x @ 0 ..= 0x17 => Ok(x.into()),
@@ -118,6 +161,7 @@ impl TypeNum {
         }
     }
 
+    #[inline]
     fn decode_u32<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u32, Error<R::Error>> {
         match self.byte & self.major_limit {
             x @ 0 ..= 0x17 => Ok(x.into()),
@@ -136,7 +180,8 @@ impl TypeNum {
         }
     }
 
-    fn decode_u64<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u64, Error<R::Error>> {
+    #[inline]
+    pub(crate) fn decode_u64<'a, R: Read<'a>>(self, reader: &mut R) -> Result<u64, Error<R::Error>> {
         match self.byte & self.major_limit {
             x @ 0 ..= 0x17 => Ok(x.into()),
             0x18 => pull_one(reader).map(Into::into),
@@ -164,6 +209,7 @@ macro_rules! decode_ux {
     ( $( $t:ty , $decode_fn:ident );* $( ; )? ) => {
         $(
             impl<'a> Decode<'a> for $t {
+                #[inline]
                 fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
                     TypeNum::new(!(major::UNSIGNED << 5), byte).$decode_fn(reader)
                 }
@@ -176,6 +222,7 @@ macro_rules! decode_nx {
     ( $( $t:ty , $decode_fn:ident );* $( ; )? ) => {
         $(
             impl<'a> Decode<'a> for types::Negative<$t> {
+                #[inline]
                 fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
                     TypeNum::new(!(major::NEGATIVE << 5), byte)
                         .$decode_fn(reader)
@@ -191,6 +238,7 @@ macro_rules! decode_ix {
     ( $( $t:ty , $decode_fn:ident );* $( ; )? ) => {
         $(
             impl<'a> Decode<'a> for $t {
+                #[inline]
                 fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
                     match byte >> 5 {
                         major::UNSIGNED => {
@@ -237,6 +285,7 @@ decode_ix! {
     i64, decode_u64;
 }
 
+#[inline]
 fn decode_x128<'a, R: Read<'a>>(name: &'static str, reader: &mut R) -> Result<[u8; 16], Error<R::Error>> {
     let byte = pull_one(reader)?;
     let len = decode_len(major::BYTES, byte, reader)?
@@ -251,6 +300,7 @@ fn decode_x128<'a, R: Read<'a>>(name: &'static str, reader: &mut R) -> Result<[u
 }
 
 impl<'a> Decode<'a> for u128 {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         if byte >> 5 == major::UNSIGNED {
             u64::decode_with(byte, reader).map(Into::into)
@@ -270,6 +320,7 @@ impl<'a> Decode<'a> for u128 {
 }
 
 impl<'a> Decode<'a> for i128 {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         match byte >> 5 {
             major::UNSIGNED => u64::decode_with(byte, reader).map(Into::into),
@@ -301,6 +352,7 @@ impl<'a> Decode<'a> for i128 {
     }
 }
 
+#[inline]
 fn decode_bytes<'a, R: Read<'a>>(name: &'static str, major_limit: u8, byte: u8, reader: &mut R)
     -> Result<&'a [u8], Error<R::Error>>
 {
@@ -321,15 +373,25 @@ fn decode_bytes<'a, R: Read<'a>>(name: &'static str, major_limit: u8, byte: u8,
     }
 }
 
+#[inline]
 #[cfg(feature = "use_alloc")]
 fn decode_buf<'a, R: Read<'a>>(major: u8, byte: u8, follow: bool, reader: &mut R, buf: &mut Vec<u8>)
-    -> Result<(), Error<R::Error>>
+    -> Result<Option<&'a [u8]>, Error<R::Error>>
 {
     const CAP_LIMIT: usize = 16 * 1024;
 
     if follow && byte == marker::BREAK {
-        Ok(())
+        Ok(None)
     } else if let Some(mut len) = decode_len(major, byte, reader)? {
+        if !follow {
+            if let Reference::Long(buf) = reader.fill(len)? {
+                if buf.len() >= len {
+                    reader.advance(len);
+                    return Ok(Some(&buf[..len]));
+                }
+            }
+        }
+
         if len <= CAP_LIMIT {
             buf.reserve(len); // TODO try_reserve ?
         }
@@ -348,7 +410,7 @@ fn decode_buf<'a, R: Read<'a>>(major: u8, byte: u8, follow: bool, reader: &mut R
             len -= readlen;
         }
 
-        Ok(())
+        Ok(None)
     } else {
         let byte = pull_one(reader)?;
         decode_buf(major, byte, true, reader, buf)
@@ -356,6 +418,7 @@ fn decode_buf<'a, R: Read<'a>>(major: u8, byte: u8, follow: bool, reader: &mut R
 }
 
 impl<'a> Decode<'a> for types::Bytes<&'a [u8]> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let buf = decode_bytes("bytes", !(major::BYTES << 5), byte, reader)?;
         Ok(types::Bytes(buf))
@@ -364,14 +427,34 @@ impl<'a> Decode<'a> for types::Bytes<&'a [u8]> {
 
 #[cfg(feature = "use_alloc")]
 impl<'a> Decode<'a> for types::Bytes<Vec<u8>> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let mut buf = Vec::new();
-        decode_buf(major::BYTES, byte, false, reader, &mut buf)?;
+        if let Some(longbuf) = decode_buf(major::BYTES, byte, false, reader, &mut buf)? {
+            buf.extend_from_slice(longbuf);
+        }
         Ok(types::Bytes(buf))
     }
 }
 
+#[cfg(feature = "use_alloc")]
+impl<'a> Decode<'a> for types::Bytes<alloc::borrow::Cow<'a, [u8]>> {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        use alloc::borrow::Cow;
+
+        let mut buf = Vec::new();
+        Ok(types::Bytes(if let Some(longbuf) = decode_buf(major::BYTES, byte, false, reader, &mut buf)? {
+            Cow::Borrowed(longbuf)
+        } else {
+            Cow::Owned(buf)
+        }))
+    }
+}
+
+
 impl<'a> Decode<'a> for &'a str {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let buf = decode_bytes("str", !(major::STRING << 5), byte, reader)?;
         core::str::from_utf8(buf).map_err(Error::InvalidUtf8)
@@ -380,16 +463,38 @@ impl<'a> Decode<'a> for &'a str {
 
 #[cfg(feature = "use_alloc")]
 impl<'a> Decode<'a> for String {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let mut buf = Vec::new();
-        decode_buf(major::STRING, byte, false, reader, &mut buf)?;
+        if let Some(longbuf) = decode_buf(major::STRING, byte, false, reader, &mut buf)? {
+            buf.extend_from_slice(longbuf);
+        }
         let buf = String::from_utf8(buf)
             .map_err(|err| Error::InvalidUtf8(err.utf8_error()))?;
         Ok(buf)
     }
 }
 
+
+#[cfg(feature = "use_alloc")]
+impl<'a> Decode<'a> for alloc::borrow::Cow<'a, str> {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        use alloc::borrow::Cow;
+
+        let mut buf = Vec::new();
+        Ok(if let Some(longbuf) = decode_buf(major::STRING, byte, false, reader, &mut buf)? {
+            Cow::Borrowed(core::str::from_utf8(longbuf).map_err(Error::InvalidUtf8)?)
+        } else {
+            let buf = String::from_utf8(buf)
+                .map_err(|err| Error::InvalidUtf8(err.utf8_error()))?;
+            Cow::Owned(buf)
+        })
+    }
+}
+
 impl<'a> Decode<'a> for types::BadStr<&'a [u8]> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let buf = decode_bytes("str", !(major::STRING << 5), byte, reader)?;
         Ok(types::BadStr(buf))
@@ -398,14 +503,34 @@ impl<'a> Decode<'a> for types::BadStr<&'a [u8]> {
 
 #[cfg(feature = "use_alloc")]
 impl<'a> Decode<'a> for types::BadStr<Vec<u8>> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let mut buf = Vec::new();
-        decode_buf(major::STRING, byte, false, reader, &mut buf)?;
+        if let Some(longbuf) = decode_buf(major::STRING, byte, false, reader, &mut buf)? {
+            buf.extend_from_slice(longbuf);
+        }
         Ok(types::BadStr(buf))
     }
 }
 
-pub fn decode_len<'a, R: Read<'a>>(major: u8, byte: u8, reader: &mut R)
+#[cfg(feature = "use_alloc")]
+impl<'a> Decode<'a> for types::BadStr<alloc::borrow::Cow<'a, [u8]>> {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        use alloc::borrow::Cow;
+
+        let mut buf = Vec::new();
+        Ok(types::BadStr(if let Some(longbuf) = decode_buf(major::STRING, byte, false, reader, &mut buf)? {
+            Cow::Borrowed(longbuf)
+        } else {
+            Cow::Owned(buf)
+        }))
+
+    }
+}
+
+#[inline]
+pub(crate) fn decode_len<'a, R: Read<'a>>(major: u8, byte: u8, reader: &mut R)
     -> Result<Option<usize>, Error<R::Error>>
 {
     if byte != (marker::START | (major << 5)) {
@@ -419,10 +544,15 @@ pub fn decode_len<'a, R: Read<'a>>(major: u8, byte: u8, reader: &mut R)
 
 #[cfg(feature = "use_alloc")]
 impl<'a, T: Decode<'a>> Decode<'a> for Vec<T> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let mut arr = Vec::new();
 
         if let Some(len) = decode_len(major::ARRAY, byte, reader)? {
+            if len <= 256 {
+                arr.reserve(len); // TODO try_reserve ?
+            }
+
             for _ in 0..len {
                 let value = T::decode(reader)?;
                 arr.push(value);
@@ -446,10 +576,15 @@ impl<'a, T: Decode<'a>> Decode<'a> for Vec<T> {
 
 #[cfg(feature = "use_alloc")]
 impl<'a, K: Decode<'a>, V: Decode<'a>> Decode<'a> for types::Map<Vec<(K, V)>> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         let mut map = Vec::new();
 
         if let Some(len) = decode_len(major::MAP, byte, reader)? {
+            if len <= 256 {
+                map.reserve(len); // TODO try_reserve ?
+            }
+
             for _ in 0..len {
                 let k = K::decode(reader)?;
                 let v = V::decode(reader)?;
@@ -473,7 +608,25 @@ impl<'a, K: Decode<'a>, V: Decode<'a>> Decode<'a> for types::Map<Vec<(K, V)>> {
     }
 }
 
+impl<'a, T: Decode<'a>> Decode<'a> for types::Tag<T> {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        let tag = TypeNum::new(!(major::TAG << 5), byte).decode_u64(reader)?;
+        let value = T::decode(reader)?;
+        Ok(types::Tag(tag, value))
+    }
+}
+
+impl<'a> Decode<'a> for types::Simple {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        let n = TypeNum::new(!(major::SIMPLE << 5), byte).decode_u8(reader)?;
+        Ok(types::Simple(n))
+    }
+}
+
 impl<'a> Decode<'a> for bool {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, _reader: &mut R) -> Result<Self, Error<R::Error>> {
         match byte {
             marker::FALSE => Ok(false),
@@ -487,6 +640,7 @@ impl<'a> Decode<'a> for bool {
 }
 
 impl<'a, T: Decode<'a>> Decode<'a> for Option<T> {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         if byte != marker::NULL && byte != marker::UNDEFINED {
             T::decode_with(byte, reader).map(Some)
@@ -496,13 +650,13 @@ impl<'a, T: Decode<'a>> Decode<'a> for Option<T> {
     }
 }
 
-#[cfg(feature = "half-f16")]
-impl<'a> Decode<'a> for half::f16 {
+impl<'a> Decode<'a> for types::F16 {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         if byte == marker::F16 {
             let mut buf = [0; 2];
             pull_exact(reader, &mut buf)?;
-            Ok(half::f16::from_be_bytes(buf))
+            Ok(types::F16(u16::from_be_bytes(buf)))
         } else {
             Err(Error::TypeMismatch {
                 name: "f16",
@@ -512,7 +666,17 @@ impl<'a> Decode<'a> for half::f16 {
     }
 }
 
+#[cfg(feature = "half-f16")]
+impl<'a> Decode<'a> for half::f16 {
+    #[inline]
+    fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
+        let types::F16(n) = types::F16::decode_with(byte, reader)?;
+        Ok(half::f16::from_bits(n))
+    }
+}
+
 impl<'a> Decode<'a> for f32 {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         if byte == marker::F32 {
             let mut buf = [0; 4];
@@ -528,6 +692,7 @@ impl<'a> Decode<'a> for f32 {
 }
 
 impl<'a> Decode<'a> for f64 {
+    #[inline]
     fn decode_with<R: Read<'a>>(byte: u8, reader: &mut R) -> Result<Self, Error<R::Error>> {
         if byte == marker::F64 {
             let mut buf = [0; 8];
